begin;

delete from ttrss_user_prefs where pref_name in ('HIDE_FEEDLIST', 'SYNC_COUNTERS', 'ENABLE_LABELS',
	'ENABLE_SEARCH_TOOLBAR', 'ENABLE_FEED_ICONS', 
	'ENABLE_OFFLINE_READING', 'EXTENDED_FEEDLIST', 'OPEN_LINKS_IN_NEW_WINDOW', 
	'USER_STYLESHEET_URL', 'ENABLE_FLASH_PLAYER');

delete from ttrss_prefs where pref_name in ('HIDE_FEEDLIST', 'SYNC_COUNTERS', 'ENABLE_LABELS',
	'ENABLE_SEARCH_TOOLBAR', 'ENABLE_FEED_ICONS', 
	'ENABLE_OFFLINE_READING', 'EXTENDED_FEEDLIST', 'OPEN_LINKS_IN_NEW_WINDOW', 
	'USER_STYLESHEET_URL', 'ENABLE_FLASH_PLAYER');

update ttrss_version set schema_version = 78;

commit;
