.flat {

	.dijitDialog .dijitDialogPaneContent {
		background : @color-panel-bg;
	}

	.dijitTab:not(.dijitTabChecked) {
		background : @color-panel-bg;
	}

	.dijitCheckBox {
		background : #ccc;
	}
}
