body.ttrss_prefs {
	background-color : @color-panel-bg;
	font-family: "Helvetica Neue", Helvetica, Arial, sans-serif;
	font-size: 14px;

	h1, h2, h3, h4 {
		font-family : @fonts-ui-bold;
		font-weight : 600;
		color : @default-text;
	}

	.dijitContentPane {
		h1:first-of-type,
		h2:first-of-type,
		h3:first-of-type {
			margin-top: 0px;
		}
	}

	#footer, #header {
		padding : 8px;
		font-size : 13px;
	}

	#header {
		float : right;
	}

	#footer_splitter {
		display : none;
	}

	#footer {
		background-color : @color-panel-bg;
		font-size : 13px;
		border : 0px;
		text-align : center;
	}

	#header img {
		vertical-align : middle;
		cursor : pointer;
	}

	.dijitTree#filterTree .dijitTreeIcon,
	.dijitTree#labelTree .dijitTreeIcon,
	.dijitTree#filterTree .dijitTreeIcon {
		display : none;
	}

	.dijitAccordionTitle i.material-icons {
		top : -1px;
		position : relative;
	}

	.dijitAccordionTitleSelected i.material-icons {
		color : white;
	}

	.dijitDialog #pref-profiles-list .dijitInlineEditBoxDisplayMode {
		padding : 0px;
	}

	div#feedlistLoading, div#filterlistLoading, div#labellistLoading {
		text-align : center;
		padding : 5px;
		color : @default-text;
	}

	div#feedlistLoading img, div#filterlistLoading img, div#labellistLoading {
		margin-right : 5px;
	}

	#errorButton {
		color : red;
	}

	.user-css-editor {
		height : 300px;
		width : 575px;
	}

	fieldset.prefs {
		min-height : 30px;
		display : flex;
		flex-direction : row;
		align-items: center;

		label:first-of-type {
			min-width : 300px;
		}

		.help-text {
			display : inline-block;
			margin-left : 10px;
		}

		.help-text-bottom {
			margin-top : 10px;
		}
	}

	fieldset.plugin {
		label.description {
			width : 600px;
			margin-right : 150px;
			display : inline-block;

			.dijitCheckBox {
				margin-right : 10px;
			}
		}
	}

	.prefErrorLog {
		tr {
			td {
				font-size: 10px;
			}

			.errno {
				font-style : italic;
				font-weight : bold;
				white-space : nowrap;
			}

			.errstr {
				word-break: break-all;
			}

			.filename, .login, .timestamp {
				color : @default-text;
			}
		}
	}

	hr {
		border-color : @border-default;
		max-width : 100%;
	}

	.phpinfo {
		table {
			border-collapse : collapse;
		}

		td.e, td.v {
			border : 1px solid #ccc;
		}

		td.e {
			font-weight : bold;
		}

		td.v {
			font-family : monospace;
			word-break : break-all;
		}
	}
}

body.ttrss_prefs,
body.ttrss_main {
	#filterNewRuleDlg {
		.dijitValidationTextAreaError  {
			background : #ffc0c0;
		}
		.dijitValidationTextArea:not(.dijitValidationTextAreaError) {
			background : #c0ffc0;
		}
	}
}

body.ttrss_prefs,
body.ttrss_utility {
	fieldset {
		border-width : 0px;
		padding : 5px 0px;
	}

	fieldset.narrow {
		padding : 2px 0px;
	}

	fieldset.align-right {
		text-align : right;
	}

	fieldset > label:first-of-type {
		min-width : 140px;
		margin-right : 20px;
		display : inline-block;
		text-align : right;
		font-weight : bold;
	}

	fieldset > label.checkbox {
		display : inline;
		font-weight : normal;
	}
}
