body.ttrss_utility.sanity_failed {
	background : #900;
}

body.ttrss_utility {
	background : @color-panel-bg;
	color : @default-fg;
	font-family: "Helvetica Neue", Helvetica, Arial, sans-serif;
	font-size: 14px;
	margin : 4em;

	.content {
		background : @default-bg;
		border : 1px solid @border-default;
		padding : 20px;
		box-shadow : 0px 1px 1px -1px rgba(0,0,0,0.1);

		h2:first-of-type {
			margin-top : 0;
		}

		h2, h3, h4 {
			color : @color-accent;
			font-family: @fonts-ui-bold;
		}

		h2 {
			font-size : 18px;
		}

		h3 {
			font-size : 16px;
		}
	}

	a {
		color : @color-link;
		text-decoration : none;
	}

	a:hover,
	a:focus {
		color: darken(@color-link, 20%);
		text-decoration: underline;
	}

	h1 {
		color : gray;
		font-family: @fonts-ui-bold;
		font-size : 18px;
		margin : 10px 0 0 0;
	}

	.footer {
		text-align : center;
		padding-top : 10px;

		a {
			color : gray;
		}

		a:hover {
			color : @color-accent;
		}
	}

	form {
		margin : 0;
	}
}

body.ttrss_utility.otp {
	.content {
		fieldset > label {
			display: inline;
		}
	}
}

body.ttrss_utility.ttrss_login {
	margin : 0;
	padding : 0;
	width : 100%;
	height : 100%;
	position : absolute;
	display : flex;
	align-items: center;
	justify-content : center;

	.container {
		max-width : 600px;
		margin-left : auto;
		margin-right : auto;

		.content {
			padding : 40px;
		}
	}
}

body.ttrss_utility.installer,
body.ttrss_utility.feed_debugger {
	margin : 2em;

	pre {
		white-space : pre-wrap;
	}
}

body.ttrss_utility.share_popup {
	margin : 0;
	padding : 0;
	background : white;

	.content {
		padding : 15px;
		border-width : 0;
		box-shadow : none;
	}
}
