@fonts-ui-bold: "Segoe WP Semibold", "Segoe UI Semibold", "Segoe UI Web Semibold", "Segoe UI", Ubuntu, "Helvetica Neue", Helvetica, Arial, sans-serif;
@fonts-ui:  "Segoe UI", Ubuntu, "Helvetica Neue", Helvetica, Arial, sans-serif;

@default-fg: black;
@default-bg: white;
@color-accent: #257aa7;
@color-accent-light: lighten(@color-accent, 50%);
@color-link: @color-accent;
@color-published: lighten(#ff5718, 10%);
@color-marked: #ffc069;
@color-accent-marked : @color-accent;
@color-panel-bg: #f5f5f5;
@color-checked: #69C671;
@border-default : #ddd;
@default-text: #555;
@color-icon: #777;
@color-tooltip-fg: @color-panel-bg;
@color-tooltip-bg: darken(@color-accent, 10%);

@embed-responsive-padding: 56.25%; // Use 56.25% for 16:9 aspect ratio, 75% for 4:3.

body.ttrss_main,
body.ttrss_prefs,
#main {
  position : absolute;
  width: 100%;
  height: 100%;
  border: 0;
  padding: 0;
  margin: 0;
}

@import "tt-rss.less";
@import "cdm.less";
@import "prefs.less";
@import "utility.less";
@import "dijit_basic.less";

@font-face {
  font-family: 'Material Icons';
  font-style: normal;
  font-weight: 400;
  src: url(../lib/iconfont/MaterialIcons-Regular.eot); /* For IE6-8 */
  src: local('Material Icons'),
  local('MaterialIcons-Regular'),
  url(../lib/iconfont/MaterialIcons-Regular.woff2) format('woff2'),
  url(../lib/iconfont/MaterialIcons-Regular.woff) format('woff'),
  url(../lib/iconfont/MaterialIcons-Regular.ttf) format('truetype');
}

.material-icons {
  font-family: 'Material Icons';
  font-weight: normal;
  font-style: normal;
  font-size: 18px;  /* Preferred icon size */
  display: inline-block;
  line-height: 1;
  //color : #777;
  text-transform: none;
  letter-spacing: normal;
  word-wrap: normal;
  white-space: nowrap;
  direction: ltr;
  vertical-align: middle;

  /* Support for all WebKit browsers. */
  -webkit-font-smoothing: antialiased;
  /* Support for Safari and Chrome. */
  text-rendering: optimizeLegibility;

  /* Support for Firefox. */
  -moz-osx-font-smoothing: grayscale;

  /* Support for IE. */
  font-feature-settings: 'liga';
}
